Package: MHD
Type: Package
Title: Metric Halfspace Depth
Version: 0.1.2
Date: 2024-01-22
Authors@R: person("Xiongtao", "Dai", email = "xiongtao.dai@hotmail.com", role = c("aut", "cre"))
Description: Metric halfspace depth for object data, generalizing Tukey's depth for Euclidean data. Implementing the method described in Dai and Lopez-Pintado (2022) <doi:10.1080/01621459.2021.2011298>.
License: GPL (>= 2)
Imports: Rcpp (>= 1.0.3), manifold, nloptr, distory, plyr
Suggests: foreach
LinkingTo: Rcpp
RoxygenNote: 7.3.0
Depends: R (>= 3.2.1)
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-01-23 16:17:10 UTC; xdai
Author: Xiongtao Dai [aut, cre]
Maintainer: Xiongtao Dai <xiongtao.dai@hotmail.com>
Repository: CRAN
Date/Publication: 2024-01-24 15:12:47 UTC
Built: R 4.4.0; x86_64-apple-darwin19.6.0; 2024-05-05 19:12:36 UTC; unix
